#include "validators.hpp"

#include <boost/program_options.hpp>
#include "CommaSeparatedOption.hpp"
#include "Mode.hpp"
#include "OutputType.hpp"
#include "OutputFormat.hpp"

namespace po = boost::program_options;

namespace harktool4
{

/** @brief Mode用バリデータ.
 * 文字列をModeに変換する.
 */
void validate(boost::any& v, const std::vector<std::string>& values, Mode* target, int)
{
    // まだvに割り当てが行われていないことをチェック（エラー時は例外を投げる）
    po::validators::check_first_occurrence(v);
    const std::string& s = po::validators::get_single_string(values);

    // string to enum
    Mode mode;
    if (s == "template") { mode = Mode::Template; }
    else if (s == "tf") { mode = Mode::TF; }
    else { throw po::invalid_option_value(s); }

    // vに領域を割り当て
    v = boost::any(mode);
}

/** @brief OutputType用バリデータ.
 * 文字列をOutputTypeに変換する.
 */
void validate(boost::any& v, const std::vector<std::string>& values, OutputType* target, int)
{
    // まだvに割り当てが行われていないことをチェック（エラー時は例外を投げる）
    po::validators::check_first_occurrence(v);
    const std::string& s = po::validators::get_single_string(values);

    // string to enum
    OutputType output_type;
    if (s == "mic") { output_type = OutputType::Mic; }
    else if (s == "noise") { output_type = OutputType::Noise; }
    else if (s == "ir") { output_type = OutputType::IR; }
    else if (s == "tsp") { output_type = OutputType::TSP; }
    else if (s == "m2pg") { output_type = OutputType::M2PG; }
    else if (s == "gtf") { output_type = OutputType::GTF; }
    else { throw po::invalid_option_value(s); }

    // vに領域を割り当て
    v = boost::any(output_type);
}

/** @brief OutputFormat用バリデータ.
 * 文字列をOutputFormatに変換する.
 */
void validate(boost::any& v, const std::vector<std::string>& values, OutputFormat* target, int)
{
    // まだvに割り当てが行われていないことをチェック（エラー時は例外を投げる）
    po::validators::check_first_occurrence(v);
    const std::string& s = po::validators::get_single_string(values);

    // string to enum
    OutputFormat output_format;
    if (s == "xml") { output_format = OutputFormat::XML; }
    else if (s == "binary") { output_format = OutputFormat::Binary; }
    else { throw po::invalid_option_value(s); }

    // vに領域を割り当て
    v = boost::any(output_format);
}

}


